<?php

namespace Drupal\crossword;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\file\FileInterface;

/**
 * Service to get structured data from Crossword file.
 *
 * This is the service you should use if you need to get all or part of the
 * crossword's structured data. While it's possible to get the data directly
 * from a crossword_parser plugin, this service is much easier to use.
 * Additionally, this service uses a caching layer and invokes an alter hook.
 */
class CrosswordDataService {

  /**
   * Cache for the result of the parse function.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The crossword file parser plugin manage.
   *
   * @var \Drupal\crossword\CrosswordFileParserManager
   */
  protected $parserManager;

  /**
   * Construct the Crossword Data Service.
   *
   * @param Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache.
   * @param Drupal\crossword\CrosswordFileParserManager $parser_manager
   *   The crossword parser manager.
   */
  public function __construct(CacheBackendInterface $cache, CrosswordFileParserManager $parser_manager) {
    $this->cache = $cache;
    $this->parserManager = $parser_manager;
  }

  /**
   * Get structured crossword data from a file.
   *
   * See CrosswordFilePluginBase::getData() for data structure.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   The structured data.
   */
  public function getData(FileInterface $file) {
    $cached = $this->cache->get($file->id());
    if (isset($cached->data['data'])) {
      $data = $cached->data['data'];
    }
    else {
      $parser = $this->parserManager->loadCrosswordFileParserFromInput($file);
      if ($parser) {
        $data = $parser->parse();
        \Drupal::moduleHandler()->alter('crossword_data', $data, $file);
        $this->cache->set($file->id(), ["data" => $data], CacheBackendInterface::CACHE_PERMANENT, $file->getCacheTags());
      }
    }
    return isset($data) ? $data : NULL;
  }

  /**
   * Get the crossword author from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return author if available.
   */
  public function getAuthor(FileInterface $file) {
    $data = $this->getData($file);
    if ($data && isset($data['author'])) {
      return $data['author'];
    }
  }

  /**
   * Get the crossword dimension from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return number of columns if available.
   */
  public function getDimensionAcross(FileInterface $file) {
    $data = $this->getData($file);
    if ($data && isset($data['puzzle']['grid'][0]) && is_array($data['puzzle']['grid'][0])) {
      return count($data['puzzle']['grid'][0]);
    }
  }

  /**
   * Get the crossword dimension from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return number of rows if available.
   */
  public function getDimensionDown(FileInterface $file) {
    $data = $this->getData($file);
    if ($data && isset($data['puzzle']['grid']) && is_array($data['puzzle']['grid'])) {
      return count($data['puzzle']['grid']);
    }
  }

  /**
   * Get the crossword dimensions from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   * @param string $delimiter
   *   The delimiter.
   *
   * @return mixed
   *   Return number of dimensions if available.
   */
  public function getDimensions(FileInterface $file, $delimiter = 'x') {
    $down = $this->getDimensionDown($file);
    if ($down) {
      $across = $this->getDimensionAcross($file);
      if ($across) {
        return $down . $delimiter . $across;
      }
    }
  }

  /**
   * Get the crossword title from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return title of crossword if available. (Not the file name!)
   */
  public function getTitle(FileInterface $file) {
    $data = $this->getData($file);
    if ($data && isset($data['title'])) {
      return $data['title'];
    }
  }

  /**
   * Find out if it's a rebus puzzle, which may require more instructions.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return bool
   *   True if it's a rebus crossword puzzle.
   */
  public function isRebus(FileInterface $file) {
    $data = $this->getData($file);
    if ($data && isset($data['puzzle']['grid'])) {
      foreach ($data['puzzle']['grid'] as $row) {
        foreach ($row as $square) {
          if ($square['rebus']) {
            return TRUE;
          }
        }
      }
    }
    return FALSE;
  }

}
