<?php

namespace Drupal\Tests\crossword\Kernel\Formatter;

use Drupal\entity_test\Entity\EntityTestRev;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\KernelTestBase;

/**
 * Base class for testing crossword field formatter plugins.
 *
 * These check that the render array gives uus what we expect.
 * See FunctionalJavascript tests for tests that test js.
 * Inspired by the core StringFormatterTest.
 */
abstract class SimpleCrosswordFormatterTestBase extends KernelTestBase {

  /**
   * The modules to load to run the test.
   *
   * @var array
   */
  public static $modules = [
    'field',
    'entity_test',
    'system',
    'user',
    'file',
    'crossword',
  ];

  /**
   * Id of the plugin.
   *
   * @var string
   */
  public $id;

  /**
   * A test entity.
   *
   * @var Drupal\entity_test\Entity\EntityTestRev
   */
  public $entity;

  /**
   * The class of the formatter.
   *
   * @var string
   */
  protected $class;

  /**
   * Settings to try in addition to default settings.
   *
   * @var array
   */
  public $customSettings;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->installConfig(['system', 'field']);
    $this->installEntitySchema('entity_test_rev');
    $this->installSchema('file', ['file_usage']);
    $this->installEntitySchema('file');
    $this->entity = $this->createTestEntity();
    $this->class = \Drupal::service('plugin.manager.field.formatter')->getDefinitions()[$this->id]['class'];
  }

  /**
   * Test the formatter using various settings.
   */
  public function testFormatter() {
    // These are the values in the test puzzle.
    $detail_values = [
      'title' => 'Test Puzzle',
      'author' => 'Test Author',
      'notepad' => 'This is a test.',
    ];
    // Get an array of settings sets together.
    $default_settings = $this->class::defaultSettings();
    $settings_array[] = $default_settings;
    if (!empty($this->customSettings)) {
      $settings_array = array_merge($settings_array, $this->customSettings);
    }

    // Test the formatter with each array of settings.
    foreach ($settings_array as $settings) {
      $render = $this->entity->field_crossword->view([
        'type' => $this->id,
        'label' => 'hidden',
        'settings' => $settings,
      ]);
      $this->assertTrue(!empty($render), "The render array is empty");
      if (isset($settings['details'])) {
        foreach ($settings['details'] as $key => $tag) {
          $detail_name = explode("_", $key)[0];
          $this->assertTrue($this->checkDetail($detail_name, $render, $detail_values[$detail_name], $tag), "The $detail_name detail is wrong.");
        }
      }
      if (isset($settings['buttons']['buttons'])) {
        foreach ($settings['buttons']['buttons'] as $button_name => $value) {
          $this->assertTrue($this->checkButton($button_name, $render, $value), "The $button_name  button is wrong.");
        }
      }
    }
  }

  /**
   * Helper function to create node that can be used for testing.
   *
   * @return Drupal\entity_test\Entity\EntityTestRev
   *   An entity to be used for testing.
   */
  protected function createTestEntity() {
    // First we move a test file to the file system.
    $contents = file_get_contents(drupal_get_path('module', 'crossword') . "/tests/files/test.txt");

    $file = file_save_data($contents, 'public://test.txt');
    $field_storage = FieldStorageConfig::create([
      'field_name' => 'field_crossword',
      'entity_type' => 'entity_test_rev',
      'type' => 'crossword',
    ]);
    $field_storage->save();

    $instance = FieldConfig::create([
      'field_storage' => $field_storage,
      'bundle' => 'entity_test_rev',
      'label' => 'Crossword',
    ]);
    $instance->save();

    $entity = EntityTestRev::create([]);
    $entity->set('field_crossword', $file->id());
    $entity->save();
    return $entity;
  }

  /**
   * Test things in the details section: title, author, notepad.
   *
   * @return bool
   *   True if everything looks good.
   */
  protected function checkDetail(string $detail_name, array $render, string $value, string $html_tag) {
    if (empty($html_tag)) {
      return empty($render[0]['#content'][$detail_name]);
    }
    return ($value == $render[0]['#content'][$detail_name]['#value']) && ($html_tag == $render[0]['#content'][$detail_name]['#tag']);
  }

  /**
   * Test the buttons.
   *
   * @return bool
   *   True if everything looks good.
   */
  protected function checkButton(string $button_name, array $render, string $value) {
    return empty($render[0]['#content']['controls']['#content'][$button_name]) xor $value == $button_name;
  }

}
